@@ -1,6 +1,6 @@
 package Classifier::MailParse;
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # MailParse.pm --- Parse a mail message or messages into words
 #
@@ -22,7 +22,7 @@ package Classifier::MailParse;
 #   along with POPFile; if not, write to the Free Software
 #   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 use strict;
 use locale;
@@ -227,7 +227,7 @@ sub new
     return $result;
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # get_color__
 #
@@ -235,7 +235,7 @@ sub new
 #
 # $word          The word to check
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub get_color__
 {
     my ( $self, $word ) = @_;
@@ -267,7 +267,7 @@ sub get_color__
     }
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # compute_rgb_distance
 #
@@ -278,7 +278,7 @@ sub get_color__
 # $left          One color
 # $right         The other color
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub compute_rgb_distance
 {
     my ( $self, $left, $right ) = @_;
@@ -305,14 +305,14 @@ sub compute_rgb_distance
     return int( sqrt( $r*$r + $g*$g + $b*$b ) );
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # compute_html_color_distance
 #
 # Calls compute_rgb_distance to set up htmlcolordistance__ from the current HTML back and
 # font colors
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub compute_html_color_distance
 {
     my ( $self ) = @_;
@@ -325,7 +325,7 @@ sub compute_html_color_distance
     $self->{htmlcolordistance__} = $self->compute_rgb_distance( $self->{htmlfontcolor__}, $self->{htmlbackcolor__} );
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # map_color
 #
@@ -333,7 +333,7 @@ sub compute_html_color_distance
 #
 # $color        A color value found in a tag
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub map_color
 {
     my ( $self, $color ) = @_;
@@ -417,7 +417,7 @@ sub map_color
     }
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # increment_word
 #
@@ -426,7 +426,7 @@ sub map_color
 #
 # $word     The word
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub increment_word
 {
     my ($self, $word) = @_;
@@ -437,7 +437,7 @@ sub increment_word
     print "--- $word ($self->{words__}{$word})\n" if ($self->{debug__});
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # update_pseudoword
 #
@@ -451,7 +451,7 @@ sub increment_word
 #
 # Returns 0 if the pseudoword was filtered out by a stopword
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub update_pseudoword
 {
     my ( $self, $prefix, $word, $encoded, $literal ) = @_;
@@ -476,7 +476,7 @@ sub update_pseudoword
     return 0;
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # update_word
 #
@@ -489,7 +489,7 @@ sub update_pseudoword
 # $prefix       A string to prefix any words with in the corpus, used for the special
 #               identification of values found in for example the subject line
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub update_word
 {
     my ($self, $word, $encoded, $before, $after, $prefix) = @_;
@@ -519,7 +519,7 @@ sub update_word
     }
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # add_line
 #
@@ -530,7 +530,7 @@ sub update_word
 # $prefix       A string to prefix any words with in the corpus, used for the special
 #               identification of values found in for example the subject line
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub add_line
 {
     my ($self, $bigline, $encoded, $prefix) = @_;
@@ -707,7 +707,7 @@ sub add_line
     }
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # update_tag
 #
@@ -719,7 +719,7 @@ sub add_line
 # $end_tag  Whether this is an end tag or not
 # $encoded  1 if this HTML was found inside encoded (base64) text
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub update_tag
 {
     my ( $self, $tag, $arg, $end_tag, $encoded ) = @_;
@@ -1101,7 +1101,7 @@ sub update_tag
     }
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # add_url
 #
@@ -1117,7 +1117,7 @@ sub update_tag
 #
 # Returns the hostname
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub add_url
 {
     my ($self, $url, $encoded, $before, $after, $prefix, $noadd) = @_;
@@ -1297,7 +1297,7 @@ sub add_url
     return $host;
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # parse_html
 #
@@ -1307,7 +1307,7 @@ sub add_url
 # $line     A line of text
 # $encoded  1 if this HTML was found inside encoded (base64) text
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub parse_html
 {
     my ( $self, $line, $encoded ) = @_;
@@ -1406,7 +1406,7 @@ sub parse_html
     return 0;
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # parse_file
 #
@@ -1419,7 +1419,7 @@ sub parse_html
 #           can be used to do multiple parses and build a single word list.  By default
 #           this is set to 1 and the word list is reset
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub parse_file
 {
     my ( $self, $file, $max_size, $reset ) = @_;
@@ -1464,7 +1464,7 @@ sub parse_file
     }
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # start_parse
 #
@@ -1475,7 +1475,7 @@ sub parse_file
 #           can be used to do multiple parses and build a single word list.  By default
 #           this is set to 1 and the word list is reset
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub start_parse
 {
     my ( $self, $reset ) = @_;
@@ -1540,14 +1540,14 @@ sub start_parse
     $self->{colorized__} .= "<tt>" if ( $self->{color__} ne '' );
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # stop_parse
 #
 # Called at the end of a parse job.  Automatically called if parse_file is used, must be
 # called after the last call to parse_line.
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub stop_parse
 {
     my ( $self ) = @_;
@@ -1575,7 +1575,7 @@ sub stop_parse
     $self->{in_html_tag__} = 0;
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # parse_line
 #
@@ -1584,7 +1584,7 @@ sub stop_parse
 #
 # $line               Line of file to parse
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub parse_line
 {
     my ( $self, $read ) = @_;
@@ -1663,7 +1663,7 @@ sub parse_line
 
                 # Append to argument if the next line begins with whitespace (isn't a new header)
 
-                if ( $line =~ /^([\t ].+)([^\r\n]+)/ ) {
+                if ( $line =~ /^([\t ])([^\r\n]+)/ ) {
                     $self->{argument__} .= "$eol$1$2";
                     next;
                 }
@@ -1759,14 +1759,14 @@ sub parse_line
     }
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # clear_out_base64
 #
 # If there's anything in the {base64__} then decode it and parse it, returns colorization
 # information to be added to the colorized output
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub clear_out_base64
 {
     my ( $self ) = @_;
@@ -1801,7 +1801,7 @@ sub clear_out_base64
     return $colorized;
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # decode_string - Decode MIME encoded strings used in the header lines
 # in email messages
@@ -1815,60 +1815,73 @@ sub clear_out_base64
 # $lang Pass in the current interface language for language specific
 # encoding conversion A B indicates base64 encoding, a Q indicates
 # quoted printable encoding
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub decode_string
-{
-    # I choose not to use "$mystring = MIME::Base64::decode( $1 );"
+{    # I choose not to use "$mystring = MIME::Base64::decode( $1 );"
     # because some spam mails have subjects like: "Subject: adjpwpekm
-    # =?ISO-8859-1?Q?=B2=E1=A4=D1=AB=C7?= dopdalnfjpw".  Therefore, it
-    # will be better to store the decoded text in a temporary variable
-    # and substitute the original string with it later. Thus, this
-    # subroutine returns the real decoded result.
+    # =?ISO-8859-1?Q?=B2=E1=A4=D1=AB=C7?= dopdalnfjpw".  Therefore we
+    # proceed along the string, from left to right, building a new
+    # string from the decoded and non-decoded parts
 
     my ( $self, $mystring, $lang ) = @_;
-
-    my $decode_it = '';
+    
     my $charset = '';
+    
+    return '' if (!defined($mystring));
 
     $lang = $self->{lang__} if ( !defined( $lang ) || ( $lang eq '' ) );
-
-    while ( $mystring =~ /=\?([\w-]+)\?(B|Q)\?(.*?)\?=/ig ) {
-        if ($2 eq "B" || $2 eq "b") {
-            $charset = $1;
-            $decode_it = decode_base64( $3 );
-
-            # for Japanese header
-            if ($lang eq 'Nihongo') {
-                $decode_it = convert_encoding( $decode_it, $charset, 'euc-jp', '7bit-jis', @{$encoding_candidates{$self->{lang__}}} );
-            }
-
-            $mystring =~ s/=\?[\w-]+\?B\?(.*?)\?=/$decode_it/i;
-        } else {
-            if ($2 eq "Q" || $2 eq "q") {
-                $decode_it = $3;
-                $decode_it =~ s/\_/=20/g;
-                $decode_it = decode_qp( $decode_it );
-
+    
+    my $output = '';
+    my $last_is_encoded = 0;
+    
+    
+    while ( $mystring =~ m/(.*?)(=\?([\w-]+)\?(B|Q)\?(.*?)\?=)/igc ) {
+        my ($pre, $atom, $encoding, $value);
+        ($pre, $atom, $charset, $encoding, $value) = ($1, $2, $3, $4, $5);
+        
+        $output .= $pre unless ($last_is_encoded && defined($atom) # Per RFC 2047 section 6.2
+                                    && $pre =~ /^[\t ]+$/);
+        
+        if (defined($atom)) {
+            if ($encoding eq "B" || $encoding eq "b") {
+                        
+                $value = decode_base64( $value );
+    
                 # for Japanese header
                 if ($lang eq 'Nihongo') {
-                    $decode_it = convert_encoding( $decode_it, $charset, 'euc-jp', '7bit-jis', @{$encoding_candidates{$self->{lang__}}} );
+                    $value = convert_encoding( $value, $charset, 'euc-jp', '7bit-jis', @{$encoding_candidates{$self->{lang__}}} );
                 }
-
-                $mystring =~ s/=\?[\w-]+\?Q\?(.*?)\?=/$decode_it/i;
+                $last_is_encoded = 1;
             }
+            elsif ($encoding eq "Q" || $encoding eq "q") {
+                $value =~ s/\_/=20/g;
+                $value = decode_qp( $value );
+    
+                # for Japanese header
+                if ($lang eq 'Nihongo') {
+                    $value = convert_encoding( $value, $charset, 'euc-jp', '7bit-jis', @{$encoding_candidates{$self->{lang__}}} );
+                }
+                $last_is_encoded = 1;
+            }
+        } else {
+            $last_is_encoded = 0;
         }
+        $output .= $value || '';
     }
-
-    return $mystring;
+    
+    # grab the unmatched tail (thanks to /gc and \G)
+    $output .= $1 if ($mystring =~ m/\G(.*)/g);
+    
+    return $output;
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # get_header - Returns the value of the from, to, subject or cc header
 #
 # $header      Name of header to return (note must be lowercase)
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub get_header
 {
     my ( $self, $header ) = @_;
@@ -1876,7 +1889,7 @@ sub get_header
     return $self->{$header . '__'} || '';
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # parse_header - Performs parsing operations on a message header
 #
@@ -1885,7 +1898,7 @@ sub get_header
 # $mime         The presently saved mime boundaries list
 # $encoding     Current message encoding
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub parse_header
 {
     my ($self, $header, $argument, $mime, $encoding) = @_;
@@ -1901,7 +1914,7 @@ sub parse_header
     $fix_argument =~ s/</&lt;/g;
     $fix_argument =~ s/>/&gt;/g;
 
-    $argument =~ s/(\r\n|\r|\n)/ /g;
+    $argument =~ s/(\r\n|\r|\n)//g;
     $argument =~ s/^[ \t]+//;
 
     if ( $self->update_pseudoword( 'header', $header, 0, $header ) ) {
@@ -2015,6 +2028,23 @@ sub parse_header
         }
     }
 
+    if ( $header =~ /^X-SpamViper-Score$/ ) {
+        # This is a header that was added by SpamViper. Works just like the
+        # SpamAssassin header.
+
+        ( my $sv_keywords = $argument ) =~ s/[\r\n]//g;
+
+        # The keywords can be found after the phrase "Mail scored X points":
+
+        $sv_keywords =~ s/Mail scored \d+ points //;
+        $sv_keywords =~ s/[\t ]//g;
+
+        foreach ( split /,/, $sv_keywords ) {
+            $self->update_pseudoword( 'spamviper', lc( $_ ), 0, $argument );
+        }
+    }
+
+
     if ( $header =~ /^X-Spam-Level$/i) {
         my $count = ( $argument =~ tr/*// );
         for ( 1 .. $count ) {
@@ -2094,7 +2124,7 @@ sub parse_header
     return ($mime, $encoding);
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # parse_css_ruleset - Parses text for CSS declarations
 #                     Uses the second part of the "ruleset" grammar
@@ -2103,7 +2133,7 @@ sub parse_header
 # $braces       1 if braces are included, 0 if excluded. Defaults to 0. Optional.
 # Returns       A hash of properties containing their expressions
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 sub parse_css_style
 {
@@ -2126,7 +2156,7 @@ sub parse_css_style
     return $hash;
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # parse_css_color - Parses a CSS color string
 #
@@ -2134,7 +2164,7 @@ sub parse_css_style
 # Returns       (r,g,b) triplet in list context, rrggbb (hex) color in scalar context
 # In case of an error: (-1,-1,-1) in list context, "error" in scalar context
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 sub parse_css_color
 {
@@ -2263,7 +2293,7 @@ sub parse_css_color
     }
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # match_attachment_filename - Matches a line  like 'attachment; filename="<filename>"
 #
@@ -2271,7 +2301,7 @@ sub parse_css_color
 # Returns       The first match (= "attchment" if found)
 #               The second match (= name of the file if found)
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub match_attachment_filename
 {
     my ( $self, $line ) = @_;
@@ -2281,7 +2311,7 @@ sub match_attachment_filename
     return ( $1, $2 );
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # file_extension - Splits a filename into name and extension
 #
@@ -2289,7 +2319,7 @@ sub match_attachment_filename
 # Returns       The name of the file
 #               The extension of the file
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub file_extension
 {
     my ( $self, $filename ) = @_;
@@ -2302,14 +2332,14 @@ sub file_extension
         return ( $filename, "" );
     }
 }
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # add_attachment_filename - Adds a file name and extension as pseudo words attchment_name
 #                         and attachment_ext
 #
 # $filename     The filename to add to the list of words
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub add_attachment_filename
 {
     my ( $self, $filename ) = @_;
@@ -2329,14 +2359,14 @@ sub add_attachment_filename
     }
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # handle_disposition - Parses Content-Disposition header to extract filename.
 #                      If filename found, at the file name and extension to the word list
 #
 # $params     The parameters of the Content-Disposition header
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub handle_disposition
 {
     my ( $self, $params ) = @_;
@@ -2348,14 +2378,14 @@ sub handle_disposition
     }
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # splitline - Escapes characters so a line will print as plain-text within a HTML document.
 #
 # $line         The line to escape
 # $encoding     The value of any current encoding scheme
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub splitline
 {
     my ( $self, $line, $encoding) = @_;
@@ -2399,7 +2429,7 @@ sub mangle
     $self->{mangle__} = $value;
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # convert_encoding
 #
@@ -2410,7 +2440,7 @@ sub mangle
 # $to           The encoding which the string is converted to
 # $default      The default encoding that is used when $from is invalid or not defined
 # @candidates   Candidate encodings for guessing
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub convert_encoding
 {
     my ( $string, $from, $to, $default, @candidates ) = @_;
@@ -2448,7 +2478,7 @@ sub convert_encoding
     return $string;
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # parse_line_with_kakasi
 #
@@ -2460,7 +2490,7 @@ sub convert_encoding
 #
 # $line          The line to be parsed
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub parse_line_with_kakasi
 {
     my ( $self, $line ) = @_;
@@ -2471,11 +2501,34 @@ sub parse_line_with_kakasi
     # Split Japanese line into words using Kakasi Wakachigaki
     # mode(-w is passed to Kakasi as argument). Both input and ouput
     # encoding are EUC-JP.
+    #
+    # Since Text::Kakasi is not thread-safe, we use it under the
+    # control of a semaphore to avoid a crash if we are running on
+    # Windows in a forked process.  
+    #
+    # Note that this requires us to detect a sub-process by looking at
+    # the value of $$.  In ActivePerl a negative PID is in a
+    # sub-process If this were to change then this code would not
+    # work.
+
+    my $need_semaphore = ( ( $^O eq 'MSWin32' ) && ( $$ < 0 ) );
+
+    if ( $need_semaphore ) {
+        if ( !defined( $self->{mutex__} ) ) {
+   	    require POPFile::Mutex;
+  	    $self->{mutex__} = new POPFile::Mutex( 'mailparse_kakasi' );
+        }
+        $self->{mutex__}->acquire();
+    }
 
     Text::Kakasi::getopt_argv("kakasi", "-w -ieuc -oeuc");
     $line = Text::Kakasi::do_kakasi($line);
     Text::Kakasi::close_kanwadict();
 
+    if ( $need_semaphore ) {
+        $self->{mutex__}->release();
+    }
+
     return $line;
 }
 
