@@ -1,8 +1,9 @@
+-- POPFILE SCHEMA 3
 -- ---------------------------------------------------------------------------------------------
 --
 -- popfile.schema - POPFile's database schema
 --
--- Copyright (c) 2001-2003 John Graham-Cumming
+-- Copyright (c) 2003-2004 John Graham-Cumming
 --
 --   This file is part of POPFile
 --
@@ -24,53 +25,67 @@
 
 -- An ASCII ERD (you might like to find the 'users' table first and work from there)
 --
---      +---------------+       +-----------------+
---      | user_template |       | bucket_template |
---      +---------------+       +-----------------+
---      |      id       |---+   |       id        |---+
---      |     name      |   |   |      name       |   |
---      |     def       |   |   |       def       |   |
---      +---------------+   |   +-----------------+   |
---                          |                         |
---      +---------------+   |     +---------------+   |
---      |  user_params  |   |     | bucket_params |   |
---      +---------------+   |     +---------------+   |
---      |      id       |   |     |      id       |   |
---  +---|    userid     |   | +---|   bucketid    |   |
---  |   |     utid      |---+ |   |     btid      |---+
---  |   |     val       |     |   |     val       |
---  |   +---------------+     |   +---------------+
---  |                         |                      +----------+
---  |                         |                      |  matrix  |      +-------+
---  |                         |   +---------+        +----------+      | words |
---  |      +----------+       |   | buckets |        |    id    |      +-------+
---  |      |   users  |       |   +---------+        |  wordid  |------|  id   |
---  |      +----------+    /--+---|    id   |=====---| bucketid |      |  word |
---  +----==|    id    |---(-------| userid  |     \  |  times   |      +-------+
---      /  |   name   |   |       |  name   |     |  | lastseen |
---      |  | password |   |       | pseudo  |     |  +----------+
---      |  +----------+   |       +---------+     |
---      |                 |                       |
---      |                 |        +-----------+  |
---      |                 |        |  magnets  |  |
---      |   +----------+  |        +-----------+  |     +--------------+
---      |   | history  |  |     +--|    id     |  |     | magnet_types |
---      |   +----------+  |     |  | bucketid  |--+     +--------------+
---      |   |   id     |  |     |  |   mtid    |--------|      id      |
---      +---| userid   |  |     |  |   val     |        |     mtype    |
---          |  frm     |  |     |  |   seq     |        |    header    |
---          |   too    |  |     |  +-----------+        +--------------+
---          |   cc     |  |     |
---          | subject  |  |     |
---          | bucketid |--+     |
---          | usedtobe |--/     |
---          | magnetid |--------+
---          |  message |
---          +----------+
+--      +---------------+         +-----------------+
+--      | user_template |         | bucket_template |
+--      +---------------+         +-----------------+
+--      |      id       |-----+   |       id        |---+
+--      |     name      |     |   |      name       |   |
+--      |     def       |     |   |       def       |   |
+--      +---------------+     |   +-----------------+   |
+--                            |                         |
+--      +---------------+     |     +---------------+   |
+--      |  user_params  |     |     | bucket_params |   |
+--      +---------------+     |     +---------------+   |
+--      |      id       |     |     |      id       |   |
+--  +---|    userid     |     | +---|   bucketid    |   |
+--  |   |     utid      |-----+ |   |     btid      |---+
+--  |   |     val       |       |   |     val       |
+--  |   +---------------+       |   +---------------+
+--  |                           |                      +----------+
+--  |                           |                      |  matrix  |      +-------+
+--  |                           |   +---------+        +----------+      | words |
+--  |      +----------+         |   | buckets |        |    id    |      +-------+
+--  |      |   users  |         |   +---------+        |  wordid  |------|  id   |
+--  |      +----------+      /--+---|    id   |=====---| bucketid |      |  word |
+--  +----==|    id    |-----(-------| userid  |     \  |  times   |      +-------+
+--      /  |   name   |     |       |  name   |     |  | lastseen |
+--      |  | password |     |       | pseudo  |     |  +----------+
+--      |  +----------+     |       +---------+     |
+--      |                   |                       |
+--      |                   |        +-----------+  |
+--      |                   |        |  magnets  |  |
+--      |   +------------+  |        +-----------+  |     +--------------+
+--      |   |   history  |  |     +--|    id     |  |     | magnet_types |
+--      |   +------------+  |     |  | bucketid  |--+     +--------------+
+--      |   |     id     |  |     |  |   mtid    |--------|      id      |
+--      +---|   userid   |  |     |  |   val     |        |     mtype    |
+--          |   hdr_from |  |     |  |   seq     |        |    header    |
+--          |   hdr_to   |  |     |  +-----------+        +--------------+
+--          |   hdr_cc   |  |     |
+--          | hdr_subject|  |     |
+--          |  bucketid  |--+     |
+--          |  usedtobe  |--/     |
+--          |  magnetid  |--------+
+--          |  hdr_date  |
+--          | inserted   |
+--          |    hash    |
+--          | committed  |
+--          |    size    |
+--          +------------+
 --
 
 -- TABLE DEFINITIONS
 
+-- ---------------------------------------------------------------------------------------------
+--
+-- popfile - data about the database
+--
+-- ---------------------------------------------------------------------------------------------
+
+create table popfile ( id integer primary key,
+                       version integer         -- version number of this schema
+                     );
+
 -- ---------------------------------------------------------------------------------------------
 --
 -- users - the table that stores the names and password of POPFile users
@@ -235,7 +250,33 @@ create table magnets( id integer primary key,    -- unique ID for this entry
                       mtid integer,              -- the magnet type
                       val varchar(255),          -- value for the magnet
                       comment varchar(255),      -- user defined comment
-                      seq int                    -- used to set the order of magnets
+                      seq integer                -- used to set the order of magnets
+                    );
+
+-- ---------------------------------------------------------------------------------------------
+--
+-- history - this table contains the items in the POPFile history that
+-- are managed by POPFile::History
+--
+-- ---------------------------------------------------------------------------------------------
+
+create table history( id integer primary key,    -- unique ID for this entry
+                      userid integer,            -- which user owns this
+                      committed integer,         -- 1 if this item has been committed
+                      hdr_from    varchar(255),  -- The From: header 
+                      hdr_to      varchar(255),  -- The To: header            
+                      hdr_cc      varchar(255),  -- The Cc: header            
+                      hdr_subject varchar(255),  -- The Subject: header
+                      hdr_date    date,          -- The Date: header
+                      hash        varchar(255),  -- MD5 message hash
+                      inserted    date,          -- When this was added
+                      bucketid integer,          -- Current classification
+                      usedtobe integer,          -- Previous classification
+                      magnetid integer,          -- If classified with magnet
+                      sort_from   varchar(255),  -- The From: header 
+                      sort_to     varchar(255),  -- The To: header            
+                      sort_cc     varchar(255),  -- The Cc: header            
+                      size        integer        -- Size of the message in Bytes
                     );
 
 -- MySQL SPECIFIC 
@@ -247,8 +288,9 @@ create table magnets( id integer primary key,    -- unique ID for this entry
 --
 -- ---------------------------------------------------------------------------------------------
 
+
 alter table buckets modify id int(11) auto_increment;
-alter table bucket_param modify id int(11) auto_increment;
+alter table bucket_params modify id int(11) auto_increment;
 alter table bucket_template modify id int(11) auto_increment;
 alter table magnets modify id int(11) auto_increment;
 alter table magnet_types modify id int(11) auto_increment;
@@ -257,8 +299,28 @@ alter table user_params modify id int(11) auto_increment;
 alter table user_template modify id int(11) auto_increment;
 alter table users modify id int(11) auto_increment;
 alter table words modify id int(11) auto_increment;
+alter table history modify id int(11) auto_increment;
+alter table popfile modify id int(11) auto_increment;
+
+-- MySQL treats char fields as case insensitive for searches, in order to have
+-- the same behavior as SQLite (case sensitive searches) we alter the word.word
+-- field to binary, that will trick MySQL into treating it the way we want.
+
 alter table words modify word binary(255);
 
+
+-- MySQL enforces types, SQLite uses the concept of manifest typing, where 
+-- the type of a value is associated with the value itself, not the column that 
+-- it is stored in. POPFile has two date fields in history where POPFile
+-- is actually storing the unix time not a date. MySQL interprets the
+-- unix time as a date of 0000-00-00, whereas SQLite simply stores the
+-- unix time integer. The follow alter table statements redefine those
+-- date fields as integer for MySQL so the correct behavior is obtained
+-- for POPFile's use of the fields.
+
+alter table history modify hdr_date int(11);
+alter table history modify inserted int(11);
+
 -- TRIGGERS
 
 -- ---------------------------------------------------------------------------------------------
@@ -274,6 +336,7 @@ alter table words modify word binary(255);
 create trigger delete_bucket delete on buckets
              begin
                  delete from matrix where bucketid = old.id;
+                 delete from history where bucketid = old.id;
                  delete from magnets where bucketid = old.id;
                  delete from bucket_params where bucketid = old.id;
              end;
@@ -288,6 +351,7 @@ create trigger delete_bucket delete on buckets
 
 create trigger delete_user delete on users
              begin
+                 delete from history where userid = old.id;
                  delete from buckets where userid = old.id;
                  delete from user_params where userid = old.id;
              end;
@@ -328,10 +392,16 @@ create trigger delete_bucket_template delete on bucket_template
 
 -- Default data
 
+-- This is schema version 3
+
+insert into popfile ( version ) values ( 3 );
+
 -- There's always a user called 'admin'
 
 insert into users ( name, password ) values ( 'admin', 'e11f180f4a31d8caface8e62994abfaf' );
 
+insert into magnets ( id, bucketid, mtid, val, comment, seq ) values ( 0, 0, 0, '', '', 0 );
+
 -- These are the possible parameters for a bucket
 --
 -- subject      1 if should do subject modification for message classified to this bucket
@@ -365,5 +435,13 @@ insert into magnet_types ( mtype, header ) values ( 'cc',      'Cc'      );
 
 insert into buckets ( name, pseudo, userid ) values ( 'unclassified', 1, 1 );
 
+-- MySQL insists that auto_increment fields start at 1. POPFile requires
+-- a special magnet record with an id of 0 in order to work properly.
+-- The following SQL statement will fix the inserted special record
+-- on MySQL installs so the id is 0, the statement should do nothing
+-- on SQLite installs since it will not satisfy the where clause.
+
+update magnets set id = 0 where id = 1 and (bucketid = 0 and mtid = 0);
+
 -- END
 
