@@ -553,7 +553,7 @@ sub commit_history__
         # sorting these fields
         #
         # "John Graham-Cumming" <spam@jgc.org> maps to
-        #     john graham-cumming <spam@jgc.org>
+        #     john graham-cumming spam@jgc.org
 
         my @sortable = ( 'from', 'to', 'cc' );
         my %sort_headers;
@@ -576,6 +576,11 @@ sub commit_history__
         my @required = ( 'from', 'to', 'cc', 'subject' );
 
         foreach my $h (@required) {
+
+            ${$header{$h}}[0] =
+                 $self->{classifier__}->{parser__}->decode_string(
+                     ${$header{$h}}[0] );
+            
             if ( !defined ${$header{$h}}[0] || ${$header{$h}}[0] =~ /^\s*$/ ) {
                 if ( $h ne 'cc' ) {
                     ${$header{$h}}[0] = "<$h header missing>";
@@ -583,10 +588,7 @@ sub commit_history__
                     ${$header{$h}}[0] = '';
                 }
             }
-
-            ${$header{$h}}[0] =
-                 $self->{classifier__}->{parser__}->decode_string(
-                     ${$header{$h}}[0] );
+            
             ${$header{$h}}[0] =~ s/\0//g;
             ${$header{$h}}[0] = $self->db__()->quote( ${$header{$h}}[0] );
         }
@@ -706,6 +708,37 @@ sub delete_slot
     $self->force_requery__();
 }
 
+#----------------------------------------------------------------------------
+#
+# start_deleting
+#
+# Called before doing a block of calls to delete_slot.  This will call
+# back into the Classifier::Bayes to tweak the database performance to
+# make this quick.
+#
+#----------------------------------------------------------------------------
+sub start_deleting
+{
+    my ( $self ) = @_;
+
+    $self->{classifier__}->tweak_sqlite( 1, 1, $self->db__() );
+}
+
+#----------------------------------------------------------------------------
+#
+# stop_deleting
+#
+# Called after doing a block of calls to delete_slot.  This will call
+# back into the Classifier::Bayes to untweak the database performance.
+#
+#----------------------------------------------------------------------------
+sub stop_deleting
+{
+    my ( $self ) = @_;
+
+    $self->{classifier__}->tweak_sqlite( 1, 0, $self->db__() );
+}
+
 #----------------------------------------------------------------------------
 #
 # get_slot_file
@@ -994,6 +1027,8 @@ sub delete_query
 {
     my ( $self, $id ) = @_;
 
+    $self->start_deleting();
+
     my $delete = $self->{queries__}{$id}{base};
     $delete =~ s/XXX/history.id/;
     my $d = $self->db__()->prepare( $delete );
@@ -1006,6 +1041,8 @@ sub delete_query
     foreach my $id (@ids) {
         $self->delete_slot( $id, 1 );
     }
+
+    $self->stop_deleting();
 }
 
 #----------------------------------------------------------------------------
