@@ -5,7 +5,7 @@ use Proxy::Proxy;
 use Digest::MD5;
 @ISA = ("Proxy::Proxy");
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # This module handles proxying the POP3 protocol for POPFile.
 #
@@ -29,7 +29,7 @@ use Digest::MD5;
 #
 #   Modified by     Sam Schinke (sschinke@users.sourceforge.net)
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 use strict;
 use warnings;
@@ -71,13 +71,13 @@ sub new
     return $self;
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # initialize
 #
 # Called to initialize the POP3 proxy module
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub initialize
 {
     my ( $self ) = @_;
@@ -111,11 +111,11 @@ sub initialize
     return $self->SUPER::initialize();
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # start
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub start
 {
     my ( $self ) = @_;
@@ -151,7 +151,7 @@ sub start
     return $self->SUPER::start();
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # child__
 #
@@ -160,7 +160,7 @@ sub start
 # $client         - an open stream to a POP3 client
 # $session        - API session key
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 sub child__
 {
     my ( $self, $client, $session ) = @_;
@@ -251,7 +251,8 @@ sub child__
                 my $ssl = defined( $options ) && ( $options =~ /ssl/i );
                 $port = 110 if ( !defined( $port ) );
 
-                if ( $mail = $self->verify_connected_( $mail, $client, $host, $port, $ssl ) )  {
+                if ( $mail = $self->verify_connected_( $mail, $client,
+                                 $host, $port, $ssl ) )  {
 
                     if ( defined( $options ) && ( $options =~ /apop/i ) ) {
 
@@ -668,7 +669,7 @@ sub child__
     $self->log_( 0, "POP3 proxy done" );
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # configure_item
 #
@@ -677,7 +678,7 @@ sub child__
 #                     when registering
 #    $language        Current language
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 sub configure_item
 {
@@ -701,7 +702,7 @@ sub configure_item
     }
 }
 
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 #
 # validate_item
 #
@@ -711,7 +712,7 @@ sub configure_item
 #    $language        The language currently in use
 #    $form            Hash containing all form items
 #
-# ---------------------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 sub validate_item
 {
